@@ -3,15 +3,15 @@
 #include "arccmt.cpp"
 
 
-Archive::Archive(RAROptions *InitCmd)
+Archive::Archive(CommandData *InitCmd)
 {
   Cmd=NULL; // Just in case we'll have an exception in 'new' below.
 
   DummyCmd=(InitCmd==NULL);
-  Cmd=DummyCmd ? (new RAROptions):InitCmd;
+  Cmd=DummyCmd ? (new CommandData):InitCmd;
 
   OpenShared=Cmd->OpenShared;
-  Format=RARFMT15;
+  Format=RARFMT_NONE;
   Solid=false;
   Volume=false;
   MainComment=false;
@@ -31,15 +31,14 @@ Archive::Archive(RAROptions *InitCmd)
   NextBlockPos=0;
 
 
-  memset(&MainHead,0,sizeof(MainHead));
-  memset(&CryptHead,0,sizeof(CryptHead));
-  memset(&EndArcHead,0,sizeof(EndArcHead));
+  MainHead.Reset();
+  CryptHead={};
+  EndArcHead.Reset();
 
   VolNumber=0;
   VolWrite=0;
   AddingFilesSize=0;
   AddingHeadersSize=0;
-  *FirstVolumeName=0;
 
   Splitting=false;
   NewArchive=false;
@@ -74,7 +73,7 @@ void Archive::CheckArc(bool EnableBroken)
 
 
 #if !defined(SFX_MODULE)
-void Archive::CheckOpen(const wchar *Name)
+void Archive::CheckOpen(const std::wstring &Name)
 {
   TOpen(Name);
   CheckArc(false);
@@ -82,7 +81,7 @@ void Archive::CheckOpen(const wchar *Name)
 #endif
 
 
-bool Archive::WCheckOpen(const wchar *Name)
+bool Archive::WCheckOpen(const std::wstring &Name)
 {
   if (!WOpen(Name))
     return false;
@@ -110,9 +109,11 @@ RARFORMAT Archive::IsSignature(const byte *D,size_t Size)
         // We check the last signature byte, so we can return a sensible
         // warning in case we'll want to change the archive format
         // sometimes in the future.
+#ifndef SFX_MODULE
         if (D[6]==0)
           Type=RARFMT15;
         else
+#endif
           if (D[6]==1)
             Type=RARFMT50;
           else
@@ -148,9 +149,9 @@ bool Archive::IsArchive(bool EnableBroken)
   }
   else
   {
-    Array<char> Buffer(MAXSFXSIZE);
+    std::vector<char> Buffer(MAXSFXSIZE);
     long CurPos=(long)Tell();
-    int ReadSize=Read(&Buffer[0],Buffer.Size()-16);
+    int ReadSize=Read(Buffer.data(),Buffer.size()-16);
     for (int I=0;I<ReadSize;I++)
       if (Buffer[I]==0x52 && (Type=IsSignature((byte *)&Buffer[I],ReadSize-I))!=RARFMT_NONE)
       {
@@ -208,8 +209,8 @@ bool Archive::IsArchive(bool EnableBroken)
       break;
   }
 
-  // This check allows to make RS based recovery even if password is incorrect.
-  // But we should not do it for EnableBroken or we'll get 'not RAR archive'
+  
+  // We should not do it for EnableBroken or we'll get 'not RAR archive'
   // messages when extracting encrypted archives with wrong password.
   if (FailedHeaderDecryption && !EnableBroken)
     return false;
@@ -231,9 +232,9 @@ bool Archive::IsArchive(bool EnableBroken)
   // first file header to set "comment" flag when reading service header.
   // Unless we are in silent mode, we need to know about presence of comment
   // immediately after IsArchive call.
-  if (HeadersLeft && (!SilentOpen || !Encrypted))
+  if (HeadersLeft && (!SilentOpen || !Encrypted) && IsSeekable())
   {
-    SaveFilePos SavePos(*this);
+    int64 SavePos=Tell();
     int64 SaveCurBlockPos=CurBlockPos,SaveNextBlockPos=NextBlockPos;
     HEADER_TYPE SaveCurHeaderType=CurHeaderType;
 
@@ -262,9 +263,10 @@ bool Archive::IsArchive(bool EnableBroken)
     CurBlockPos=SaveCurBlockPos;
     NextBlockPos=SaveNextBlockPos;
     CurHeaderType=SaveCurHeaderType;
+    Seek(SavePos,SEEK_SET);
   }
   if (!Volume || FirstVolume)
-    wcsncpyz(FirstVolumeName,FileName,ASIZE(FirstVolumeName));
+    FirstVolumeName=FileName;
 
   return true;
 }
@@ -300,7 +302,7 @@ uint Archive::FullHeaderSize(size_t Size)
 
 
 #ifdef USE_QOPEN
-bool Archive::Open(const wchar *Name,uint Mode)
+bool Archive::Open(const std::wstring &Name,uint Mode)
 {
   // Important if we reuse Archive object and it has virtual QOpen
   // file position not matching real. For example, for 'l -v volname'.
@@ -335,3 +337,23 @@ int64 Archive::Tell()
 }
 #endif
 
+
+// Return 0 if dictionary size is invalid. If size is RAR7 only, return
+// the adjusted nearest bottom value. Return header flags in Flags.
+uint64 Archive::GetWinSize(uint64 Size,uint &Flags)
+{
+  Flags=0;
+  // Allow 128 KB - 1 TB range.
+  if (Size<0x20000 || Size>0x10000000000ULL)
+    return 0;
+  uint64 Pow2=0x20000; // Power of 2 dictionary size.
+  for (;2*Pow2<=Size;Pow2*=2)
+    Flags+=FCI_DICT_BIT0;
+  if (Size==Pow2)
+    return Size;  // If 'Size' is the power of 2, return it as is.
+
+  // Get the number of Pow2/32 to add to Pow2 for nearest value not exceeding 'Size'.
+  uint64 Fraction=(Size-Pow2)/(Pow2/32);
+  Flags+=(uint)Fraction*FCI_DICT_FRACT0;
+  return Pow2+Fraction*(Pow2/32);
+}
